@@ -6,13 +6,18 @@ Structure schemas for different items in SQLite database
 import json
 import uuid
 import time
+import bcrypt
 
 from datetime import datetime
 
 from sqlalchemy import (
-    not_, func, select, create_engine, update,
-    Column, String, Boolean,ForeignKey, JSON)
-from sqlalchemy.orm import declarative_base, sessionmaker, mapped_column, Session, scoped_session
+    not_, or_, and_, func, select, insert, event, union_all, create_engine, update, exc,
+    Column, String, Boolean,ForeignKey, JSON, text)
+from sqlalchemy.orm import (
+    declarative_base, sessionmaker, 
+    mapped_column, Session, scoped_session,
+    with_loader_criteria
+)
 from sqlalchemy.pool import NullPool
 
 from typing import Generator
@@ -21,20 +26,35 @@ from shapely.geometry import box, shape
 
 from typing_extensions import Union
 
-from .models import Base, User, VisSession, Item, Layer, Structure, ImageOverlay, Annotation
+from .models import (
+    Base, User, UserAccess, 
+    VisSession, Item, Layer, 
+    LocalItem, RemoteItem,
+    Structure, ImageOverlay, Annotation,
+    Data
+)
 
 
 TABLE_NAMES = {
     'user': User,
-    'visSession': VisSession,
+    'user_access': UserAccess,
+    'vis_session': VisSession,
     'item': Item,
+    'local_item': LocalItem,
+    'remote_item': RemoteItem,
     'layer': Layer,
     'structure': Structure,
     'image_overlay': ImageOverlay,
-    'annotation': Annotation
+    'annotation': Annotation,
+    'data': Data
 }
 
 
+#TODO: add access check to UserAccess table for any query that mentions data from an item
+# This includes Item, Layer, Structure, ImageOverlay, and Annotation elements
+# either user_id or user_token can be used to identify a user, though user_id is a column in UserAccess
+
+
 class fusionDB:
     def __init__(self,
                  db_url:str,
@@ -53,6 +73,20 @@ class fusionDB:
         Base.metadata.create_all(bind = self.engine)
         self.SessionLocal = scoped_session(sessionmaker(bind=self.engine))
 
+        """
+        # Not sure how to add this as an automatic filterer, this will prevent non-public Items from being available on "select" statements
+        @event.listens_for(self.SessionLocal,"do_orm_execute")
+        def _do_orm_execute(orm_execute_state):
+            if (
+                orm_execute_state.is_select
+                and not orm_execute_state.is_column_load
+                and not orm_execute_state.is_relationship_load
+            ):
+                orm_execute_state.statement = orm_execute_state.statement.options(
+                    with_loader_criteria(entity_or_base = Item, where_criteria = Item.public == True)
+                )
+        """
+
     @contextmanager
     def get_db(self) -> Generator[Session, None, None]:
         # Generator type has types, yield, send, return (in this case it yields a Session, sends None and returns None)
@@ -80,9 +114,31 @@ class fusionDB:
             session.add(obj)
 
         return True
+
+    def remove(self, obj, session = None):
+        # Using sessionmaker in a context manager
+        # (closes automatically and rolls back in the event of a database error)
+        # Testing if the session object also has a .delete method
+        if session is None:
+            with self.get_db() as session:
+                session.delete(obj)
+                session.commit()
+        else:
+            session.delete(obj)
+        
+        return True
     
     def get_create(self, table_name:str, inst_id:Union[str,None] = None, kwargs:Union[dict,None] = None):
-        
+        """Function for updating or adding items to the database based on whether or not that particular instance already exists
+
+        :param table_name: Name of table that the instance belongs/should belong to
+        :type table_name: str
+        :param inst_id: A unique id assigned to the new/updated instance, defaults to None
+        :type inst_id: Union[str,None], optional
+        :param kwargs: Additional arguments used in the creation/updation of an instance, defaults to None
+        :type kwargs: Union[dict,None], optional
+        :return: Returns the newly created instance if the table name exists 
+        """
         with self.get_db() as session:
             if table_name in TABLE_NAMES:
                 if not inst_id is None:
@@ -127,10 +183,58 @@ class fusionDB:
             else:
                 return None
 
+    def get_remove(self, table_name:str, inst_id:Union[str,None] = None, user_id: Union[str,None] = None, vis_session_id: Union[str,None] = None):
+        """The opposite of self.get_create, checks if an item is present in the table and if it is, deletes it.
+
+        :param table_name: Name of table that the instance belongs to
+        :type table_name: str
+        :param inst_id: A unique id assigned to the instance, defaults to None
+        :type inst_id: Union[str,None], optional
+        :return: Returns True if the removal was successful/the table name exists and that instance is in the table
+        """
+        with self.get_db() as session:
+            if table_name in TABLE_NAMES:
+                if not inst_id is None:
+                    get_remove_result = session.query(
+                        TABLE_NAMES.get(table_name)
+                    ).filter_by(id = inst_id)
+
+                    if not get_remove_result:
+                        #print("Instance not present in db")
+                        # If this thing isn't in the table, don't do anything
+                        pass
+                    else:
+                        # This is if this thing DOES exist in the table, remove it
+                        if not user_id is None:
+                            get_remove_result = get_remove_result.filter(
+                                getattr(TABLE_NAMES.get(table_name),'user')==user_id
+                            )
+
+                        if not vis_session_id is None:
+                            get_remove_result = get_remove_result.filter(
+                                getattr(TABLE_NAMES.get(table_name),'session')==vis_session_id
+                            )
+
+                        if get_remove_result:
+                            for r in get_remove_result.all():
+                                self.remove(r,session)
+                        else:
+                            #print('Not found for this user/vis_session')
+                            pass
+                else:
+                    # Don't delete anything if no instance id is provided
+                    get_remove_result = None
+
+                session.commit()
+
+                return get_remove_result
+            else:
+                return None
+
     def count(self, table_name:str):
         """Get the count of unique instances within this specific table
 
-        :param table_name: Name of table to query (user, visSession, item, layer, structure, image_overlay, annotation)
+        :param table_name: Name of table to query (user, vis_session, item, layer, structure, image_overlay, annotation)
         :type table_name: str
         :return: Count of instances
         :rtype: int
@@ -236,6 +340,9 @@ class fusionDB:
         if not search_kwargs.get('type','') in TABLE_NAMES:
             return []
 
+        # By default, not validating user access and only searching public items
+        public_only_search = True
+
         #print(f'db search called: {json.dumps(search_kwargs,indent=4)}')
         with self.get_db() as session:
             search_query = session.query(
@@ -245,11 +352,27 @@ class fusionDB:
             # Applying filters
             search_filters = search_kwargs.get('filters')
             if not search_filters is None:
-                
                 table_joins = list(set(list(search_filters.keys())))
+
+                # Determine ahead of time if will need to check user access
+                if search_kwargs.get('type')=='item' and 'user' in table_joins:
+                    public_only_search = False
+
+                for t in table_joins:
+                    if t in TABLE_NAMES:
+                        if hasattr(TABLE_NAMES.get(t),'item') and 'user' in table_joins:
+                            public_only_search = False
+
+                if search_kwargs.get('type')=='item' and public_only_search:
+                    # This covers the case of public-only-searches
+                    search_query = search_query.filter(Item.public == True)
+                    
                 for t in table_joins:
                     if t in TABLE_NAMES:
                         if not t == search_kwargs.get('type'):
+                            # Don't join on User, save that for checking public_only_search
+                            if t=='user':
+                                continue
                             search_query = search_query.join(TABLE_NAMES.get(t))
 
                         for k,v in search_filters.get(t).items():
@@ -260,6 +383,20 @@ class fusionDB:
                             elif type(v)==dict:
                                 search_query = self.search_op(search_query,t,k,v)
 
+            if not public_only_search:
+                # Combining original query with all public items and all items the user has access to
+                user_args = search_filters.get('user')
+                query_user = self.get_user(
+                    user_id = user_args.get('id'),
+                    user_token = user_args.get('token')
+                )
+                if not query_user is None:
+                    # Filtering public or has access
+                    search_query = search_query.filter(or_(Item.public==True,Item.user_access.any(id = query_user.get('id'))))
+                else:
+                    # Filtering public
+                    search_query = search_query.filter(Item.public==True)
+
             return_list = []
             for idx,i in enumerate(search_query.all()):
                 if not size is None:
@@ -269,28 +406,37 @@ class fusionDB:
                 if idx>=offset:
                     return_list.append(i.to_dict())
 
-            #print(f'n returned from db_search: {len(return_list)}')
-            
             return return_list
       
-    def add_slide(self, slide_id:str, slide_name:str, metadata: dict, image_metadata: dict, image_filepath:Union[str,None], annotations_metadata:dict,annotations:Union[list,dict,None]):
+    def add_slide(self,
+        slide_id:str, 
+        slide_name:str, 
+        metadata: dict, 
+        image_metadata: dict, 
+        image_filepath:Union[str,None], 
+        annotations_metadata:dict,
+        annotations:Union[list,dict,None],
+        vis_session_id: Union[str,None] = None,
+        user_id: Union[str,None] = None,
+        public: bool = False):
 
+        #print(f'{slide_id=}, {public=}')
         new_item = self.get_create(
-            table_name = 'item',
+            table_name = 'local_item',
             inst_id = slide_id,
             kwargs = {
                 'name': slide_name,
                 'meta': metadata,
                 'image_meta': image_metadata,
                 'ann_meta': annotations_metadata,
-                'filepath': image_filepath
+                'filepath': image_filepath,
+                'session': vis_session_id,
+                'public': public, 
             }
         )
 
-        print('fusionDB.add_slide:')
-        print(new_item)
-        print(slide_id)
-        print(f'len(annotations): {len(annotations)}')
+        if not public and user_id is not None:
+            self.add_access(slide_id,user_id)
 
         for ann_idx, ann in enumerate(annotations):
             # Adding layer
@@ -299,10 +445,9 @@ class fusionDB:
                 inst_id = ann['properties']['_id'],
                 kwargs = {
                     'name': ann['properties']['name'],
-                    'item': slide_id
+                    'item': slide_id,
                 }
             )
-            print(f'new_layer: {new_layer}')
 
             if 'image_path' in ann:
                 # Adding image overlay layer
@@ -313,7 +458,8 @@ class fusionDB:
                         'bounds': ann['image_bounds'],
                         'properties': ann['image_properties'],
                         'image_src': ann['image_path'],
-                        'layer': ann['properties']['_id']
+                        'layer': ann['properties']['_id'],
+                        'item': slide_id
                     }
                 )
             else:
@@ -321,61 +467,149 @@ class fusionDB:
                 for f_idx, f in enumerate(ann['features']):
                     new_structure = self.get_create(
                         table_name = 'structure',
-                        inst_id = f['properties']['_id'],
+                        inst_id = f['properties'].get('_id',uuid.uuid4().hex[:24]),
                         kwargs = {
                             'geom': f['geometry'],
                             'properties': f['properties'],
                             'layer': ann['properties']['_id'],
-                            'item': slide_id
+                            'item': slide_id,
                         }
                     )
 
     def add_vis_session(self, vis_session: dict):
-        #TODO: Adding new visualization session to database (local should just be the same each time so don't have to back it up)
-        # Contains {"current": [], "local":[], "data": {}, "current_user": {}, "session": {}}
 
-        #TODO: Guest user sessions all start with "guestuser"
         vis_session_kwargs = {
-            'user': vis_session.get('current_user',{'_id': f'guestuser'+self.get_uuid()[:15]}).get('_id'),
+            'user': vis_session.get('user',{}).get('id'),
             'data': vis_session.get('data',{})
         }
 
         new_vis_session = self.get_create(
-            table_name = 'visSession',
+            table_name = 'vis_session',
             inst_id = vis_session.get('session',{}).get('id'),
             kwargs = vis_session_kwargs
         )
 
-        #TODO: Adding items in "current"
-        for c in vis_session.get('current',[]):
-            print(f'add_vis_session current slide: {json.dumps(c,indent=4)}')
-            item_kwargs = {
-                'name': c.get('name'),
-                'meta': c.get('meta'),
-                'image_meta': c.get('image_meta'),
-                'ann_meta': c.get('ann_meta'),
-                'filepath': c.get('filepath'),
-                'session': vis_session.get('session',{}).get('id')
+        new_user = self.get_create(
+            table_name = 'user',
+            inst_id = vis_session.get('user',{}).get('id'),
+            kwargs = {
+                k:v
+                for k,v in vis_session.get('user',{}).items() 
+                if not k=='id'
             }
-            new_item = self.get_create(
-                table_name = 'item',
-                inst_id = c.get('id'),
-                kwargs = c
+        )
+
+    def add_access(self, item_id, user_id):
+
+        with self.get_db() as session:
+            statement = session.execute(
+                insert(UserAccess).values(
+                    user_id = user_id,
+                    item_id = item_id
+                )
             )
+        
+    def get_user(self, user_token:Union[str,None] = None, user_id: Union[str,None] = None) -> User:
 
-            #TODO: Get Layers, Structures, ImageOverlays, and Annotations? Or wait to add those to the database?
-            # Should duplicates be added? It would be important to preserve different versions of each Layer/Structure/etc. if changes are made.
-            # The default/static version of annotations can just be the source (local file/DSA annotations/item/{id})
+        with self.get_db() as session:
+            if not user_token is None:
+                search_query = session.execute(
+                    select(User).where(User.token==user_token)
+                ).first()
+            elif not user_id is None:
+                search_query = session.execute(
+                    select(User).where(User.id==user_id)
+                ).first()
 
-    def get_names(self, table_name:str, size:Union[int,None]=None, offset = 0):
+            if not search_query is None:
+                if len(search_query)>0:
+                    return search_query[0].to_dict()
+                else:
+                    return None
+            else:
+                return None
 
+    def check_user_login_password(self, user_login: str, user_password: str):
+
+        with self.get_db() as session:
+            search_query = session.execute(
+                select(User).where(User.login==user_login)
+            ).first()
+
+            if not search_query is None:
+                if len(search_query)>0:
+                    if search_query[0].verify_password(user_password):
+                        return search_query[0].to_dict()
+                    else:
+                        return None
+                else:
+                    return None
+            else:
+                return None
+
+    def create_new_user(self, user_kwargs: dict):
+
+        # Required kwargs are login, password, firstname, lastname
+        if not all([user_kwargs.get(j) for j in ['login','password','firstName','lastName']]):
+            return None
+        else:
+            # Check uniqueness of login
+            with self.get_db() as session:
+                search_query = session.execute(
+                    select(User).where(User.login==user_kwargs.get('login'))
+                ).first()
+
+                if search_query is None:
+                    
+                    user_kwargs['password'] = bcrypt.hashpw(user_kwargs['password'].encode(),bcrypt.gensalt())
+                    user_kwargs['token'] = self.get_uuid()
+                    user_kwargs['updated'] = datetime.now()
+
+                    new_id = self.get_uuid()
+                    new_user = User(id = new_id, **user_kwargs)
+                    user_kwargs['id'] = new_id
+
+                    self.add(new_user, session)
+                    session.commit()
+
+                    return user_kwargs
+                else:
+                    return None
+
+    def check_user_access(self, user_id:str, admin:bool = False) -> list:
+
+        user_access_rows = []
+        with self.get_db() as session:
+            user_access = session.query(UserAccess)
+
+            for i in user_access.all():
+                user_access_rows.append(i)
+
+        # Admins have access to everything
+        non_public_access_list = []
+        if admin:
+            non_public_access_list =  [i[1] for i in user_access_rows]
+        else:
+            for user,item in user_access_rows:
+                if user_id==user:
+                    non_public_access_list.append(item)
+
+        return non_public_access_list
+
+    def get_names(self, table_name:str, user_token: Union[str,None] = None, size:Union[int,None]=None, offset = 0):
+        
+        #TODO: This should get access controlled
         return_names = []
         with self.get_db() as session:
 
             if table_name in TABLE_NAMES:
+                #if not table_name=='item':
                 name_query = session.execute(
                     select(getattr(TABLE_NAMES.get(table_name),'name'))
                 )
+                #else:
+                #    name_query = session.query(Item.name)
+                #    name_query = name_query.join(UserAccess)
 
                 for a_idx,a in enumerate(name_query.all()):
                     if not size is None:
@@ -386,6 +620,26 @@ class fusionDB:
 
             return return_names
 
+    def get_ids(self, table_name: str, user_token: Union[str,None] = None, size:Union[int,None] = None, offset = 0):
+
+        #TODO: This should be access controlled
+        return_ids = []
+        with self.get_db() as session:
+
+            if table_name in TABLE_NAMES:
+                id_query = session.execute(
+                    select(getattr(TABLE_NAMES.get(table_name),'id'))
+                )
+
+                for a_idx, a in enumerate(id_query.all()):
+                    if not size is None:
+                        if len(return_ids)==size:
+                            break
+                    if a_idx>=offset:
+                        return_ids.append(a[0])
+            
+            return return_ids
+
     async def get_item_annotations(self, item_id:str, user_id:Union[str,None] = None, vis_session_id:Union[str,None] = None)->list:
         """Loading annotations from item database
 
@@ -411,8 +665,6 @@ class fusionDB:
                 }
             }
         )
-        print('fusionDB.get_item_annotations:')
-        print(item_layers)
 
         for l in item_layers:
             layer_name = l.get('name')
@@ -587,10 +839,45 @@ class fusionDB:
                 if shape(structure_geom).intersects(query_box):
                     return_list.append(structure_id)
 
-
-            print(f'Time for get_structures_in_bbox: {time.time() - start}')
-
-
             return return_list
 
+    def get_structure_generator(self, item_id: Union[str,list,None] = None, layer_id:Union[str,list,None] = None, structure_id: Union[str,list,None] = None):
+
+
+        with self.get_db() as session:
+            search_query = session.query(
+                Structure.id,
+                Structure.geom,
+                Structure.properties
+            ).filter(Structure.layer == Layer.id).filter(Layer.item==Item.id)
+
+            if not item_id is None:
+                if type(item_id)==list:
+                    search_query = search_query.filter(Item.id.in_(item_id))
+                elif type(item_id)==str:
+                    search_query = search_query.filter(Item.id == item_id)
+
+            if not layer_id is None:
+                if type(layer_id)==list:
+                    search_query = search_query.filter(Layer.id.in_(layer_id))
+                elif type(layer_id)==str:
+                    search_query = search_query.filter(Layer.id==layer_id)
+
+            if not structure_id is None:
+                if type(structure_id)==list:
+                    search_query = search_query.filter(Structure.id.in_(structure_id))
+                elif type(structure_id)==str:
+                    search_query = search_query.filter(Structure.id==structure_id)
+            
+            #returned_props = ['structure.id','geometry','structure.properties']
+            #return_list = []
+            #for idx,i in enumerate(search_query.all()):
+            #    i_dict = {'_index': idx}
+            #    for prop,prop_name in zip(i,returned_props):
+            #        i_dict[prop_name] = prop
+
+            #    return_list.append(i_dict)
+
+            #return return_list
+            return search_query.all()
 
