@@ -4,11 +4,12 @@ Tile server components
 
 """
 import os
-from fastapi import FastAPI, APIRouter, Response
+from fastapi import FastAPI, APIRouter, Response, Request, Depends
 from fastapi.middleware.cors import CORSMiddleware
 import large_image
 import requests
 import json
+from typing import Annotated
 from typing_extensions import Union
 
 from copy import deepcopy
@@ -37,12 +38,14 @@ class Slide:
                  image_filepath: Union[str,None] = None,
                  annotations: Union[str,list,dict,None] = None,
                  metadata: Union[dict,None] = None,
-                 image_style: Union[dict,None] = None):
+                 image_style: Union[dict,None] = None,
+                 public: bool = False):
         
         self.image_filepath = image_filepath
         self.annotations = annotations
         self.metadata = metadata
         self.image_style = image_style
+        self.public = public
 
         # Checking path exists
         assert(os.path.exists(self.image_filepath))
@@ -174,7 +177,10 @@ class Slide:
 class TileServer:
     """Components which pull information from a slide(s)
     """
-    pass
+    
+    def get_slide_urls(slide_dict,**kwargs):
+        raise NotImplementedError
+
 
 class LocalTileServer(TileServer):
     """Tile server from image saved locally. Uses large-image to read and parse image formats (default: [common])
@@ -184,14 +190,10 @@ class LocalTileServer(TileServer):
                  tile_server_port:int = 8050,
                  host: str = 'localhost',
                  protocol: str = 'http',
-                 cors_options: dict = {'origins': ['*'], 'allow_methods': ['*'], 'allow_headers': ['*'], 'expose_headers': ['*'], 'max_age': '36000000'}
+                 jupyter_server_url: Union[str,None] = None,
+                 cors_options: dict = {}
                  ):
         """Constructor method
-
-        :param local_image_path: File path for image saved locally
-        :type local_image_path: str
-        :param tile_server_port: Tile server path where tiles are accessible from, defaults to '8050'
-        :type tile_server_port: str, optional
         """
 
         self.database = database
@@ -199,65 +201,97 @@ class LocalTileServer(TileServer):
         self.host = host
         self.protocol = protocol
         self.cors_options = cors_options
+        self.jupyter_server_url = jupyter_server_url
 
-        self.app = FastAPI()
+        if self.jupyter_server_url is None or self.jupyter_server_url=='':
+            self.access_url = f'{self.protocol}://{self.host}:{self.tile_server_port}'
+        else:
+            self.access_url = self.jupyter_server_url
+
+        self.app = FastAPI(
+            title = 'TileServer',
+            description = 'Locally deployed tileserver for high-resolution microscopy images',
+            version = '3.6.42'
+        )
+
+        self.app.add_middleware(
+            CORSMiddleware,
+            allow_origins=self.cors_options.get('allow_origins',['*']),
+            allow_methods=self.cors_options.get('allow_methods',['GET','OPTIONS']),
+            allow_headers = self.cors_options.get('allow_headers',['*']),
+            expose_headers = self.cors_options.get('expose_headers',['*']),
+            allow_credentials = self.cors_options.get('allow_credentials',False)
+        )
 
         self.router = APIRouter()
         self.router.add_api_route('/',self.root,methods=["GET","OPTIONS"])
+        self.router.add_api_route('/ids',self.get_ids,methods=["GET","OPTIONS"])
         self.router.add_api_route('/names',self.get_names,methods=["GET","OPTIONS"])
+        self.router.add_api_route('/{id}/info',self.get_id_info,methods=["GET","OPTIONS"])
         self.router.add_api_route('/{id}/tiles/{z}/{x}/{y}',self.get_tile,methods=["GET","OPTIONS"])
         self.router.add_api_route('/{id}/image_metadata',self.get_image_metadata,methods=["GET","OPTIONS"])
         self.router.add_api_route('/{id}/metadata',self.get_metadata,methods=["GET","OPTIONS"])
         self.router.add_api_route('/{id}/tiles/region',self.get_region,methods=["GET","OPTIONS"])
         self.router.add_api_route('/{id}/tiles/thumbnail',self.get_thumbnail,methods=["GET","OPTIONS"])
         self.router.add_api_route('/{id}/annotations',self.get_annotations,methods=["GET","OPTIONS"])
-        self.router.add_api_route('/{id}/annotations/metadata',self.get_annotations_metadata,methods=["GET","OPTIONS"])
+        self.router.add_api_route('/{id}/annotations/metadata',self.get_annotations_metadata,methods=["GET", "OPTIONS"])
         self.router.add_api_route('/{id}/annotations/data/list',self.get_annotations_property_keys,methods=["GET","OPTIONS"])
         self.router.add_api_route('/{id}/annotations/data',self.get_annotations_property_data,methods=["GET","OPTIONS"])
 
     def __str__(self):
         return f'TileServer class to {self.host}:{self.tile_server_port}'
-
-    def __len__(self):
-        """Return number of items 
-
-        :return: Count of items in local database
-        :rtype: int
-        """
-        count = self.database.count(
-            table_name = 'item'
-        )
-
-        return count
     
-    def add_new_image(self,new_image_id:str, new_image_path:str, new_annotations:Union[str,list,dict,None] = None, new_metadata:Union[dict,None] = None, new_image_style:Union[dict,None] = None):
+    def add_new_image(self,
+        new_image_id:str, 
+        new_image_path:str, 
+        new_annotations:Union[str,list,dict,None] = None, 
+        new_metadata:Union[dict,None] = None, 
+        new_image_style:Union[dict,None] = None,
+        new_image_public: bool = False,
+        session_id: Union[str,None] = None,
+        user_id: Union[str,None] = None
+        ):
 
         # Verifying filepaths and loading annotations
         new_local_item = Slide(
             image_filepath=new_image_path,
             annotations = new_annotations,
             metadata = new_metadata,
-            image_style=new_image_style
+            image_style=new_image_style,
+            public = new_image_public
         )
 
-        slide_name = new_image_path.split(os.sep)[-1]
+        self.add_new_slide(new_local_item,session_id,user_id)
+
+    def add_new_slide(self, slide_id: str, slide_obj: Slide, session_id: Union[str,None] = None, user_id: Union[str,None] = None):
+
+        slide_name = slide_obj.image_filepath.split(os.sep)[-1]
         # Adding information to database
         self.database.add_slide(
-            new_image_id,
-            slide_name,
-            new_local_item.metadata,
-            new_local_item.image_metadata,
-            new_local_item.image_filepath,
-            new_local_item.annotations_metadata,
-            new_local_item.processed_annotations
+            slide_id = slide_id,
+            slide_name = slide_name,
+            metadata = slide_obj.metadata,
+            image_metadata = slide_obj.image_metadata,
+            image_filepath = slide_obj.image_filepath,
+            annotations_metadata = slide_obj.annotations_metadata,
+            annotations = slide_obj.processed_annotations,
+            vis_session_id = session_id,
+            user_id = user_id,
+            public = slide_obj.public
         )
 
-        #print(f"Item names in database: {self.database.get_names('item')}")
+        # If this is not a public slide, add to UserAccess table
+        #if not slide_obj.public:
+
+        #    self.database.add_access(
+        #        item_id = slide_id,
+        #        user_id = user_id
+        #    )
 
     def root(self):
         return {'message': "Oh yeah, now we're cooking"}
 
-    async def get_item(self,item_id:str):
+    async def get_item(self,item_id:str, token:Union[str,None] = None):
         """Grabbing item from database
 
         :param item_id: String uuid for local item
@@ -265,20 +299,30 @@ class LocalTileServer(TileServer):
         :return: Item instance
         :rtype: None
         """
+
+        if not token is None:
+            user_filter = {
+                'user': {
+                    'token': token
+                }
+            }
+        else:
+            user_filter = {}
+
         image_item = await self.database.search(
             search_kwargs = {
                 'type': 'item',
                 'filters': {
                     'item': {
                         'id': item_id
-                    }
-                }
+                    } 
+                }| user_filter
             }
         )
 
         return image_item
     
-    async def get_tile_source(self,item_id:str,style:Union[str,None]=None):
+    async def get_tile_source(self,item_id:str,style:Union[str,None]=None, token: Union[str,None] = None):
         """Getting large-image tile source for a given id+style combo. 
 
         :param item_id: String uuid for local image
@@ -288,7 +332,8 @@ class LocalTileServer(TileServer):
         :return: Tile source
         :rtype: None
         """
-        image_item = await asyncio.gather(self.get_item(item_id))
+
+        image_item = await asyncio.gather(self.get_item(item_id,token))
         if len(image_item[0])==0:
             return None
         else:
@@ -302,12 +347,20 @@ class LocalTileServer(TileServer):
 
             return tile_source
         
-    async def get_item_annotations(self, item_id:str):
+    async def get_item_annotations(self, item_id:str, request: Request = None):
         """Loading annotations from item database
 
         :param item_id: String uuid for local image
         :type item_id: str
         """
+        user_filter = {}
+        if not request is None:
+            if request.query_params.get('token'):
+                user_filter = {
+                    'user': {
+                        'token': request.query_params.get('token')
+                    }
+                }
 
         item_annotations = []
 
@@ -318,7 +371,7 @@ class LocalTileServer(TileServer):
                     'item': {
                         'id': item_id
                     }
-                }
+                } | user_filter
             }
         )
 
@@ -326,14 +379,15 @@ class LocalTileServer(TileServer):
             layer_name = l.get('name')
             layer_id = l.get('id')
 
+            #TODO: Check if this also needs the user token
             layer_structures = await self.database.search(
                 search_kwargs={
                     'type': 'structure',
                     'filters': {
                         'layer': {
                             'id': layer_id
-                        }
-                    }
+                        } 
+                    }| user_filter
                 }
             )
 
@@ -356,6 +410,7 @@ class LocalTileServer(TileServer):
                     }
                 )
             else:
+                #TODO: Check if this needs a user_token argument
                 # This could be an ImageOverlay layer
                 image_overlays = await self.database.search(
                     search_kwargs = {
@@ -364,7 +419,7 @@ class LocalTileServer(TileServer):
                             'layer': {
                                 'id': layer_id
                             }
-                        }
+                        } | user_filter
                     }
                 )
 
@@ -377,17 +432,45 @@ class LocalTileServer(TileServer):
 
         return item_annotations
 
-    def get_names(self):
+    def get_names(self, request: Request):
         """Get names of items in fusionDB
 
         :return: Message dictionary containing list of all item names
         :rtype: dict
         """
+
         item_names = self.database.get_names(
-            table_name = 'item'
+            table_name = 'item',
         )
 
         return {'message': item_names}
+
+    def get_ids(self):
+        """Get all available ids in the database
+        """
+        item_ids = self.database.get_ids(
+            table_name = 'item'
+        )
+
+        return {'message': item_ids}
+    
+    async def get_id_info(self, id:str):
+        """Get info for a single id
+        """
+        image_item = await asyncio.gather(self.get_item(id))
+
+        if len(image_item[0])>0:
+            item_info = image_item[0][0]
+            return Response(
+                content = json.dumps(item_info,default=str),
+                media_type = 'application/json'
+            )
+        else:
+            return Response(
+                content = 'invalid image id',
+                media_type='application/json',
+                status_code=400,
+            )
     
     def get_item_names_ids(self, filters = None, size = None, offset = 0):
         """Get list of names and ids of all locally stored images in this tileserver
@@ -412,31 +495,70 @@ class LocalTileServer(TileServer):
 
         return item_names_ids
 
+    @staticmethod
+    def get_slide_urls(slide_dict,user_token,standalone = False):
+        
+        slide_id = slide_dict.get('id')
+        url = slide_dict.get('url')
+
+        if not standalone:
+            slide_url_dict = {
+                'tiles': f'{url}/tileserver/{slide_id}/tiles/'+'{z}/{x}/{y}',
+                'regions': f'{url}/tileserver/{slide_id}/tiles/region',
+                'image_metadata': f'{url}/tileserver/{slide_id}/image_metadata',
+                'metadata': f'{url}/tileserver/{slide_id}/metadata',
+                'annotations': f'{url}/tileserver/{slide_id}/annotations',
+                'annotations_metadata':f'{url}/tileserver/{slide_id}/annotations/metadata',
+                'annotations_region':f'{url}/tileserver/{slide_id}/annotations'
+            }
+        else:
+            slide_url_dict = {
+                'tiles': f'{url}/{slide_id}/tiles/'+'{z}/{x}/{y}',
+                'regions': f'{url}/{slide_id}/tiles/region',
+                'image_metadata': f'{url}/{slide_id}/image_metadata',
+                'metadata': f'{url}/{slide_id}/metadata',
+                'annotations': f'{url}/{slide_id}/annotations',
+                'annotations_metadata':f'{url}/{slide_id}/annotations/metadata',
+                'annotations_region':f'{url}/{slide_id}/annotations'
+            }
+
+        if not user_token is None:
+            for k,v in slide_url_dict.items():
+                v += f'?token={user_token}'
+
+        return slide_url_dict
+
     def get_tiles_url(self,slide_id):
         tiles_url = f'{self.protocol}://{self.host}:{self.tile_server_port}/{slide_id}/tiles/'+'{z}/{x}/{y}'
+
         return tiles_url
 
     def get_regions_url(self,slide_id):
         regions_url = f'{self.protocol}://{self.host}:{self.tile_server_port}/{slide_id}/tiles/region'
+
         return regions_url
 
     def get_annotations_url(self,slide_id):
         annotations_url = f'{self.protocol}://{self.host}:{self.tile_server_port}/{slide_id}/annotations'
+
         return annotations_url
 
     def get_annotations_metadata_url(self,slide_id):
         annotations_metadata_url = f'{self.protocol}://{self.host}:{self.tile_server_port}/{slide_id}/annotations/metadata'
+
         return annotations_metadata_url
 
     def get_metadata_url(self,slide_id):
         metadata_url = f'{self.protocol}://{self.host}:{self.tile_server_port}/{slide_id}/metadata'
+
         return metadata_url
         
     def get_image_metadata_url(self,slide_id):
         image_metadata_url = f'{self.protocol}://{self.host}:{self.tile_server_port}/{slide_id}/image_metadata'
+
         return image_metadata_url
 
-    async def get_tile(self,id:str,z:int, x:int, y:int, style:Union[None,str] = None):
+    async def get_tile(self,id:str,z:int, x:int, y:int, style:Union[None,str] = None, request:Request = None):
         """Tiles endpoint, returns an image tyle based on provided coordinates
 
         :param id: Local item id
@@ -452,10 +574,24 @@ class LocalTileServer(TileServer):
         :return: Image tile containing bytes encoded pixel information
         :rtype: Response
         """
-        tile_source = await asyncio.gather(self.get_tile_source(id,style))
+
+        # large-image getTile doesn't do anything with negative zoom or tile coordinates
+        if any([i<0 for i in [z,x,y]]):
+            return Response(status_code=200)
+
+        token = None
+        if not request is None:
+            if request.query_params.get('token'):
+                token = request.query_params.get('token')
+
+        tile_source = await asyncio.gather(self.get_tile_source(id,style, token))
         tile_source = tile_source[0]
         if tile_source is None:
-            return Response(content = 'invalid image id', media_type='application/json',status_code=400)
+            return Response(
+                content = 'invalid image id', 
+                media_type='application/json',
+                status_code=400,
+            )
 
         tile_metadata = tile_source.getMetadata()
 
@@ -476,37 +612,78 @@ class LocalTileServer(TileServer):
                 dtype=np.uint8
             ).tobytes()
 
-        return Response(content = raw_tile, media_type='image/png')
+        return Response(
+            content = raw_tile, 
+            media_type='image/png',
+        )
     
-    async def get_image_metadata(self,id:str):
+    async def get_image_metadata(self,id:str, request:Request = None):
         """Getting large-image metadata for image
 
         :return: Dictionary containing metadata for local image
         :rtype: Response
         """
 
-        image_item = await asyncio.gather(self.get_item(id))
+        token = None
+        if not request is None:
+            if request.query_params.get('token'):
+                token = request.query_params.get('token')
+
+
+        image_item = await asyncio.gather(self.get_item(id,token))
         if len(image_item[0])>0:
             image_meta = image_item[0][0].get('image_meta',{})
-            return Response(content = json.dumps(image_meta),media_type = 'application/json')
+            if not image_meta is None:
+                return Response(
+                    content = json.dumps(image_meta),
+                    media_type = 'application/json',
+                )
+            else:
+                return Response(
+                    content = json.dumps({}),
+                    media_type = 'application/json',
+                )
         else:
-            return Response(content = 'invalid image id',media_type='application/json', status_code=400)
+            return Response(
+                content = 'invalid image id',
+                media_type='application/json', 
+                status_code=400,
+            )
         
-    async def get_metadata(self, id:str):
+    async def get_metadata(self, id:str, request: Request = None):
         """Getting metadata associated with slide/case/patient
 
         :param image: Index of local image
         :type image: int
         """
-        image_item = await asyncio.gather(self.get_item(id))
+
+        token = None
+        if not request is None:
+            if request.query_params.get('token'):
+                token = request.query_params.get('token')
+
+        image_item = await asyncio.gather(self.get_item(id, token))
 
         if len(image_item[0])>0:
             item_meta = image_item[0][0].get('meta',{})
-            return Response(content = json.dumps(item_meta),media_type = 'application/json')
+            if not item_meta is None:
+                return Response(
+                    content = json.dumps(item_meta),
+                    media_type = 'application/json',
+                )
+            else:
+                return Response(
+                    content = json.dumps({}),
+                    media_type = 'application/json',
+                )
         else:
-            return Response(content = 'invalid image id',media_type='application/json',status_code=400)
+            return Response(
+                content = 'invalid image id',
+                media_type='application/json',
+                status_code=400,
+            )
     
-    def get_region(self, id:str, top: int, left: int, bottom:int, right:int,style:Union[None,str] = None):
+    async def get_region(self, id:str, top: int, left: int, bottom:int, right:int,style:Union[None,str] = None):
         """
         Grabbing a specific region in the image based on bounding box coordinates
         """
@@ -515,10 +692,20 @@ class LocalTileServer(TileServer):
         :return: Image region (bytes encoded)
         :rtype: Response
         """
-        tile_source = self.get_tile_source(id,style)
+        token = None
+        if not request is None:
+            if request.query_params.get('token'):
+                token = request.query_params.get('token')
 
+        tile_source = await asyncio.gather(self.get_tile_source(id,style, token))
+        tile_source = tile_source[0]
+        
         if tile_source is None:
-            return Response(content = 'invalid image id', media_type = 'application/json', status_code = 400)
+            return Response(
+                content = 'invalid image id',
+                media_type = 'application/json',
+                status_code = 400,
+            )
 
         image_region, mime_type = tile_source.getRegion(
             region = {
@@ -529,25 +716,40 @@ class LocalTileServer(TileServer):
             },
         )
 
-        return Response(content = image_region, media_type = 'image/png')
+        return Response(
+            content = image_region,
+            media_type = 'image/png',
+        )
 
-    def get_thumbnail(self, id:str, style:Union[None,str] = None):
+    async def get_thumbnail(self, id:str, style:Union[None,str] = None, request: Request = None):
         """Grabbing an image thumbnail
 
-        :param image: _description_
-        :type image: int
+        :param id: Unique id for locally stored item
+        :type id: str
         """
+        token = None
+        if not request is None:
+            if request.query_params.get('token'):
+                token = request.query_params.get('token')
 
-        tile_source = self.get_tile_source(id,style)
+        tile_source = await asyncio.gather(self.get_tile_source(id,style, token))
+        tile_source = tile_source[0]
 
         if tile_source is None:
-            return Response(content = 'invalid image index', media_type = 'application/json', status_code=400)
+            return Response(
+                content = 'invalid image id', 
+                media_type = 'application/json', 
+                status_code=400,
+            )
 
         thumbnail, mime_type = tile_source.getThumbnail(encoding='PNG')
         
-        return Response(content = thumbnail, media_type = 'image/png')
+        return Response(
+            content = thumbnail, 
+            media_type = 'image/png',
+        )
 
-    async def get_annotations(self,id:str, top:Union[int,None]=None, left:Union[int,None]=None, bottom: Union[int,None]=None, right: Union[int,None]=None):
+    async def get_annotations(self,id:str, top:Union[int,None]=None, left:Union[int,None]=None, bottom: Union[int,None]=None, right: Union[int,None]=None, request: Request = None):
         """Getting annotations for a given item id, optionally specifying a region within which to grab annotations.
 
         :param id: String uuid for local image
@@ -563,14 +765,14 @@ class LocalTileServer(TileServer):
         :return: Annotations for item (optionally within a specified region)
         """
 
-        image_annotations = await asyncio.gather(self.get_item_annotations(id))
+        image_annotations = await asyncio.gather(self.get_item_annotations(id, request))
         
         if len(image_annotations)>0:
             if all([i is None for i in [top,left,bottom,right]]):
                 # Returning all annotations by default
                 return Response(
                     content = json.dumps(image_annotations),
-                    media_type='application/json'
+                    media_type='application/json',
                 )
             else:
                 # Parsing region of annotations:
@@ -624,24 +826,31 @@ class LocalTileServer(TileServer):
                             print(f'Unrecognized annotation format found for image: {id}')
                     return Response(
                         content = json.dumps(image_region_anns), 
-                        media_type='application/json'
+                        media_type='application/json',
                     )
 
         else:
             return Response(
                 content = 'invalid image id',
                 media_type = 'application/json', 
-                status_code = 400
+                status_code = 400,
             )
 
-    async def get_annotations_metadata(self,id:str):
+    async def get_annotations_metadata(self,id:str, request: Request):
         """Getting metadata for annotations for an item
 
         :param id: String uuid for locally stored image
         :type id: str
         :return: Metadata associated with annotations for that image
         """
-        image_item = await asyncio.gather(self.get_item(id))
+
+        token = None
+        if not request is None:
+            if request.query_params.get('token'):
+                token = request.query_params.get('token')
+
+
+        image_item = await asyncio.gather(self.get_item(id, token))
         if len(image_item[0])==0:
             return Response(
                 content = 'invalid image id',
@@ -652,19 +861,20 @@ class LocalTileServer(TileServer):
         image_item = image_item[0][0]
 
         ann_meta = image_item.get('ann_meta',[])
+
         return Response(
             content = json.dumps(ann_meta),
-            media_type='application/json'
+            media_type = 'application/json'
         )
 
-    async def get_annotations_property_keys(self,id:str):
+    async def get_annotations_property_keys(self,id:str, request: Request):
         """Getting the names of properties stored in an image's annotations
 
         :param id: String uuid for locally stored image
         :type id: int
         """
         #print(f'id from get_annotations_property_keys: {id}')
-        image_anns = await self.get_item_annotations(id)
+        image_anns = await self.get_item_annotations(id, request)
         property_list = []
         property_names = []
         for a in image_anns:
@@ -770,10 +980,10 @@ class LocalTileServer(TileServer):
         return Response(
             content = json.dumps(property_list),
             media_type='application/json',
-            status_code=200
+            status_code=200,
         )
 
-    async def get_annotations_property_data(self,id:str,include_keys:Union[str,None] = None,include_anns:Union[str,None] = None):
+    async def get_annotations_property_data(self,id:str,include_keys:Union[str,None] = None,include_anns:Union[str,None] = None, request: Request = None):
         """Getting data from annotations of specified image, attempting to mirror output of https://github.com/girder/large_image/blob/master/girder_annotation/girder_large_image_annotation/utils/__init__.py
 
         :param id: String uuid for locally stored image.
@@ -783,9 +993,15 @@ class LocalTileServer(TileServer):
         :param include_anns: Which annotations to include (name/id or __all__ or list)
         :type include_anns: Union[str,list,None]
         """
+
+        token = None
+        if not request is None:
+            if request.query_params.get('token'):
+                token = request.query_params.get('token')
+
                 
-        image_item = await self.get_item(id)[0]
-        image_anns = await self.get_item_annotations(id)
+        image_item = await self.get_item(id, token)[0]
+        image_anns = await self.get_item_annotations(id,request)
 
         include_keys = include_keys.split(',')
 
@@ -862,7 +1078,7 @@ class LocalTileServer(TileServer):
         return Response(
             content = json.dumps({'data': data_list, 'columns': include_keys}),
             media_type='application/json',
-            status_code=200
+            status_code=200,
         )
 
     def start(self):
@@ -876,12 +1092,11 @@ class LocalTileServer(TileServer):
         # Enabling CORS (https://fastapi.tiangolo.com/tutorial/cors/#use-corsmiddleware)
         self.app.add_middleware(
             CORSMiddleware,
-            allow_origins=self.cors_options['origins'],
-            allow_methods=self.cors_options['allow_methods'],
-            allow_credentials = True,
-            allow_headers=self.cors_options['allow_headers'],
-            expose_headers=self.cors_options['expose_headers'],
-            max_age=self.cors_options['max_age']
+            allow_origins=self.cors_options.get('allow_origins',['*']),
+            allow_methods=self.cors_options.get('allow_methods',['GET','OPTIONS']),
+            allow_headers = self.cors_options.get('allow_headers',['*']),
+            expose_headers = self.cors_options.get('expose_headers',['*']),
+            allow_credentials = self.cors_options.get('allow_credentials',False)
         )
 
         uvicorn.run(self.app,host=self.host,port=self.tile_server_port)
@@ -940,10 +1155,49 @@ class DSATileServer(TileServer):
 
         self.annotations_region_url = f'{api_url}/annotation/'
 
-
     def __str__(self):
         return f'DSATileServer for {self.base_url}'
 
+    @staticmethod
+    def get_slide_urls(slide_dict,user_token):
+        
+        item_id = slide_dict.get('remote_id')
+        item_url = slide_dict.get('url')
+
+        slide_url_dict = {
+            'tiles': f'{item_url}/item/{item_id}/tiles/zxy/'+'{z}/{x}/{y}',
+            'regions': f'{item_url}/item/{item_id}/tiles/region',
+            'image_metadata': f'{item_url}/item/{item_id}/tiles',
+            'metadata': f'{item_url}/item/{item_id}',
+            'annotations': f'{item_url}/annotation/item/{item_id}',
+            'annotations_metadata':f'{item_url}/annotation?itemId={item_id}',
+            'annotations_region':f'{item_url}/annotation'
+        }
+
+        if not user_token is None:
+            for k,v in slide_url_dict.items():
+                if not '?' in v:
+                    v += f'?token={user_token}'
+                else:
+                    v += f'&token={user_token}'
+
+        # Adding annotations_geojson_url?
+        annotations_metadata = requests.get(slide_url_dict.get('annotations_metadata')).json()
+
+        annotations_geojson_urls = []
+        for a in annotations_metadata:
+            req_str = f'{item_url}/annotation/{a.get("_id")}/geojson'
+            if not user_token is None:
+                req_str += f'?token={user_token}'
+
+            annotations_geojson_urls.append(
+                req_str
+            )
+
+        slide_url_dict['annotations_geojson_url'] = annotations_geojson_urls
+
+        return slide_url_dict
+
 class CustomTileServer(TileServer):
     """CustomTileServer component if using some other tiles endpoint (must pass tileSize and levels in dictionary)
     """
@@ -975,4 +1229,5 @@ class CustomTileServer(TileServer):
     def __str__(self):
         return f'CustomTileServer for {self.tiles_url}'
 
-
+    def get_slide_urls(self, slide_dict):
+        pass
